# coding: utf-8


import pytest  # NOQA


class TestNone:
    def test_dump00(self):
        import srsly.ruamel_yaml  # NOQA

        data = None
        s = srsly.ruamel_yaml.round_trip_dump(data)
        assert s == "null\n...\n"
        d = srsly.ruamel_yaml.round_trip_load(s)
        assert d == data

    def test_dump01(self):
        import srsly.ruamel_yaml  # NOQA

        data = None
        s = srsly.ruamel_yaml.round_trip_dump(data, explicit_end=True)
        assert s == "null\n...\n"
        d = srsly.ruamel_yaml.round_trip_load(s)
        assert d == data

    def test_dump02(self):
        import srsly.ruamel_yaml  # NOQA

        data = None
        s = srsly.ruamel_yaml.round_trip_dump(data, explicit_end=False)
        assert s == "null\n...\n"
        d = srsly.ruamel_yaml.round_trip_load(s)
        assert d == data

    def test_dump03(self):
        import srsly.ruamel_yaml  # NOQA

        data = None
        s = srsly.ruamel_yaml.round_trip_dump(data, explicit_start=True)
        assert s == "---\n...\n"
        d = srsly.ruamel_yaml.round_trip_load(s)
        assert d == data

    def test_dump04(self):
        import srsly.ruamel_yaml  # NOQA

        data = None
        s = srsly.ruamel_yaml.round_trip_dump(
            data, explicit_start=True, explicit_end=False
        )
        assert s == "---\n...\n"
        d = srsly.ruamel_yaml.round_trip_load(s)
        assert d == data



# coding: utf-8

"""
Testing copy and deepcopy, instigated by Issue 84 (Peter Amstutz)
"""

import copy

import pytest  # NOQA

from .roundtrip import dedent, round_trip_load, round_trip_dump


class TestDeepCopy:
    def test_preserve_flow_style_simple(self):
        x = dedent(
            """\
        {foo: bar, baz: quux}
        """
        )
        data = round_trip_load(x)
        data_copy = copy.deepcopy(data)
        y = round_trip_dump(data_copy)
        print("x [{}]".format(x))
        print("y [{}]".format(y))
        assert y == x
        assert data.fa.flow_style() == data_copy.fa.flow_style()

    def test_deepcopy_flow_style_nested_dict(self):
        x = dedent(
            """\
        a: {foo: bar, baz: quux}
        """
        )
        data = round_trip_load(x)
        assert data["a"].fa.flow_style() is True
        data_copy = copy.deepcopy(data)
        assert data_copy["a"].fa.flow_style() is True
        data_copy["a"].fa.set_block_style()
        assert data["a"].fa.flow_style() != data_copy["a"].fa.flow_style()
        assert data["a"].fa._flow_style is True
        assert data_copy["a"].fa._flow_style is False
        y = round_trip_dump(data_copy)

        print("x [{}]".format(x))
        print("y [{}]".format(y))
        assert y == dedent(
            """\
        a:
          foo: bar
          baz: quux
        """
        )

    def test_deepcopy_flow_style_nested_list(self):
        x = dedent(
            """\
        a: [1, 2, 3]
        """
        )
        data = round_trip_load(x)
        assert data["a"].fa.flow_style() is True
        data_copy = copy.deepcopy(data)
        assert data_copy["a"].fa.flow_style() is True
        data_copy["a"].fa.set_block_style()
        assert data["a"].fa.flow_style() != data_copy["a"].fa.flow_style()
        assert data["a"].fa._flow_style is True
        assert data_copy["a"].fa._flow_style is False
        y = round_trip_dump(data_copy)

        print("x [{}]".format(x))
        print("y [{}]".format(y))
        assert y == dedent(
            """\
        a:
        - 1
        - 2
        - 3
        """
        )


class TestCopy:
    def test_copy_flow_style_nested_dict(self):
        x = dedent(
            """\
        a: {foo: bar, baz: quux}
        """
        )
        data = round_trip_load(x)
        assert data["a"].fa.flow_style() is True
        data_copy = copy.copy(data)
        assert data_copy["a"].fa.flow_style() is True
        data_copy["a"].fa.set_block_style()
        assert data["a"].fa.flow_style() == data_copy["a"].fa.flow_style()
        assert data["a"].fa._flow_style is False
        assert data_copy["a"].fa._flow_style is False
        y = round_trip_dump(data_copy)
        z = round_trip_dump(data)
        assert y == z

        assert y == dedent(
            """\
        a:
          foo: bar
          baz: quux
        """
        )

    def test_copy_flow_style_nested_list(self):
        x = dedent(
            """\
        a: [1, 2, 3]
        """
        )
        data = round_trip_load(x)
        assert data["a"].fa.flow_style() is True
        data_copy = copy.copy(data)
        assert data_copy["a"].fa.flow_style() is True
        data_copy["a"].fa.set_block_style()
        assert data["a"].fa.flow_style() == data_copy["a"].fa.flow_style()
        assert data["a"].fa._flow_style is False
        assert data_copy["a"].fa._flow_style is False
        y = round_trip_dump(data_copy)

        print("x [{}]".format(x))
        print("y [{}]".format(y))
        assert y == dedent(
            """\
        a:
        - 1
        - 2
        - 3
        """
        )



# coding: utf-8

import functools
import catalogue

from ._version import version
from .exceptions import *
from ._packer import Packer as _Packer
from ._unpacker import unpackb as _unpackb
from ._unpacker import unpack as _unpack
from ._unpacker import Unpacker as _Unpacker
from ._ext_type import ExtType
from ._msgpack_numpy import encode_numpy as _encode_numpy
from ._msgpack_numpy import decode_numpy as _decode_numpy


msgpack_encoders = catalogue.create("srsly", "msgpack_encoders", entry_points=True)
msgpack_decoders = catalogue.create("srsly", "msgpack_decoders", entry_points=True)

msgpack_encoders.register("numpy", func=_encode_numpy)
msgpack_decoders.register("numpy", func=_decode_numpy)


# msgpack_numpy extensions
class Packer(_Packer):
    def __init__(self, *args, **kwargs):
        default = kwargs.get("default")
        for encoder in msgpack_encoders.get_all().values():
            default = functools.partial(encoder, chain=default)
        kwargs["default"] = default
        super(Packer, self).__init__(*args, **kwargs)


class Unpacker(_Unpacker):
    def __init__(self, *args, **kwargs):
        object_hook = kwargs.get("object_hook")
        for decoder in msgpack_decoders.get_all().values():
            object_hook = functools.partial(decoder, chain=object_hook)
        kwargs["object_hook"] = object_hook
        super(Unpacker, self).__init__(*args, **kwargs)


def pack(o, stream, **kwargs):
    """
    Pack an object and write it to a stream.
    """
    packer = Packer(**kwargs)
    stream.write(packer.pack(o))


def packb(o, **kwargs):
    """
    Pack an object and return the packed bytes.
    """
    return Packer(**kwargs).pack(o)


def unpack(stream, **kwargs):
    """
    Unpack a packed object from a stream.
    """
    if "object_pairs_hook" not in kwargs:
        object_hook = kwargs.get("object_hook")
        for decoder in msgpack_decoders.get_all().values():
            object_hook = functools.partial(decoder, chain=object_hook)
        kwargs["object_hook"] = object_hook
    return _unpack(stream, **kwargs)


def unpackb(packed, **kwargs):
    """
    Unpack a packed object.
    """
    if "object_pairs_hook" not in kwargs:
        object_hook = kwargs.get("object_hook")
        for decoder in msgpack_decoders.get_all().values():
            object_hook = functools.partial(decoder, chain=object_hook)
        kwargs["object_hook"] = object_hook
    return _unpackb(packed, **kwargs)


# alias for compatibility to simplejson/marshal/pickle.
load = unpack
loads = unpackb

dump = pack
dumps = packb
